@@ -1,6 +1,6 @@
 /*
  * ascpu is the CPU statistics monitor utility for X Windows
- * Copyright (c) 1998-2000  Albert Dorofeev <Albert@tigr.net>
+ * Copyright (c) 1998-2005  Albert Dorofeev <albert@tigr.net>
  * For the updates see http://www.tigr.net/
  *
  * This software is distributed under GPL. For details see LICENSE file.
@@ -87,6 +87,10 @@ static struct nlist	nlst[] = {
 };
 #endif
 
+#ifdef	__linux__
+char	proc_extended = 0;
+#endif
+
 #ifdef _AIX32   /* AIX > 3.1 */
 /* Kernel memory file */
 #define KMEM_FILE "/dev/kmem"
@@ -461,6 +465,52 @@ void prepare_format()
 {
 	int i;
 	scan_format[0] = '\0';
+#ifdef __linux__
+	int fd, len, major, minor, micro;
+	if ((fd = open("/proc/sys/kernel/osrelease", O_RDONLY)) == -1) {
+		printf("ascpu : cannot open /proc/sys/kernel/osrelease\n");
+		exit(1);
+	}
+	if ((len = read(fd, buffer, sizeof(buffer)-1)) < 0) {
+		printf("ascpu : cannot read /proc/sys/kernel/osrelease\n");
+		exit(1);
+	}
+	buffer[len] = '\0';
+	close(fd);
+	if (!sscanf(buffer, "%d.%d.%d", &major, &minor, &micro)) {
+		printf("ascpu : cannot get Linux Major/Minor version\n");
+		exit(1);
+	}
+	if (major > 2 || (major == 2 && minor > 6) || (major == 2 && minor == 6 && micro >= 10)) {
+#ifdef DEBUG
+		printf("Linux Kernel >= 2.6.10 detected\n");
+#endif
+		proc_extended = 2;
+	} else if (major == 2 && minor >= 5) {
+#ifdef DEBUG
+		printf("Linux Kernel >= 2.5 and < 2.6.10 detected\n");
+#endif
+		proc_extended = 1;
+	} else {
+		proc_extended = 0;
+#ifdef DEBUG
+		printf("Linux Kernel < 2.5 detected\n");
+#endif
+	}
+	if ( state.cpu_number < 0 ) {
+		sprintf(scan_format, proc_extended?((proc_extended-1)?"cpu %%ld %%ld %%ld %%ld %%*ld %%*ld %%*ld %%*ld\n":"cpu %%ld %%ld %%ld %%ld %%*ld %%*ld %%*ld\n"):"cpu %%ld %%ld %%ld %%ld\n");
+	} else {
+		sprintf(scan_format, proc_extended?((proc_extended-1)?"cpu %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld\n":"cpu %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld\n"):"cpu %%*ld %%*ld %%*ld %%*ld\n");
+		for (i=0; i<state.cpu_number; ++i) {
+			sprintf(&scan_format[strlen(scan_format)], 
+				proc_extended?((proc_extended-1)?"cpu%d %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld\n":"cpu%d %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld %%*ld\n"):"cpu%d %%*ld %%*ld %%*ld %%*ld\n",
+				i);
+		}
+		sprintf(&scan_format[strlen(scan_format)], 
+			proc_extended?((proc_extended-1)?"cpu%d %%ld %%ld %%ld %%ld %%*ld %%*ld %%*ld %%*ld\n":"cpu%d %%ld %%ld %%ld %%ld %%*ld %%*ld %%*ld\n"):"cpu%d %%ld %%ld %%ld %%ld\n",
+			state.cpu_number);
+	}
+#else
 	if ( state.cpu_number < 0 ) {
 		sprintf(scan_format, "cpu %%ld %%ld %%ld %%ld\n");
 	} else {
@@ -474,6 +524,7 @@ void prepare_format()
 				"cpu%d %%ld %%ld %%ld %%ld\n",
 				state.cpu_number);
 	}
+#endif
 #ifdef DEBUG
 	printf("The format is [%s]\n", scan_format);
 #endif
@@ -936,6 +987,9 @@ void ascpu_initialize(
                 mainDisplay,    /* display */
                 iconWindow,     /* window */
                 ExposureMask    /* event_mask */
+#ifdef FOR_CLICK
+		| ButtonPressMask
+#endif
                 );
                 
         /* Creating Graphics context */
